/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_X86_LINKAGE_H
#define _ASM_X86_LINKAGE_H

#include <linux/stringify.h>
#include <asm/cpufeatures.h>

#undef notrace
#define notrace __attribute__((no_instrument_function))

#ifdef CONFIG_X86_32
#define asmlinkage CPP_ASMLINKAGE __attribute__((regparm(0)))
#endif /* CONFIG_X86_32 */

#ifdef CONFIG_X86_32
#define cond_syscall(x)	asm(				\
	".weak " __stringify(x) "\n\t"			\
	".set  " __stringify(x) ","			\
		 __stringify(__ia32_sys_ni_syscall))
#else
#define cond_syscall(x)	asm(				\
	".weak " __stringify(x) "\n\t"			\
	".set  " __stringify(x) ","			\
		 __stringify(__x64_sys_ni_syscall))
#endif

#ifdef CONFIG_ARCH_HAS_SYSCALL_WRAPPER
#define SYSCALL_ALIAS(a, name)				\
	__visible typeof(name) a __attribute__((alias(__stringify(name))))
#endif

#define __ALIGN		.balign CONFIG_FUNCTION_ALIGNMENT, 0xcc;
#define __ALIGN_STR	__stringify(__ALIGN)
#define __ALIGN_EXEC	.p2align 4, 0x90
#define __ALIGN_EXEC_STR __stringify(__ALIGN_EXEC)

#define ASM_FUNC_ALIGN		__ALIGN_STR
#define __FUNC_ALIGN		__ALIGN
#define SYM_F_ALIGN		__FUNC_ALIGN

#ifdef __ASSEMBLY__

#if defined(CONFIG_RETHUNK) && !defined(__DISABLE_EXPORTS) && !defined(BUILD_VDSO) && !defined(_SETUP)
#define RET	.section ".return_sites","a",%progbits; .long 202406f - .; .previous; 202406: pax_jmp __x86_return_thunk
#elif defined(CONFIG_SLS)
#define RET	ret; int3
#else /* !CONFIG_SLS */
#define RET	ret
#endif /* CONFIG_RETHUNK */

#else /* __ASSEMBLY__ */

#if defined(CONFIG_RETHUNK) && !defined(__DISABLE_EXPORTS) && !defined(BUILD_VDSO) && !defined(_SETUP)
#define ASM_RET	".section \".return_sites\",\"a\",%progbits; .long 202406f - .; .previous; 202406: pax_jmp __x86_return_thunk"
#elif defined(CONFIG_SLS)
#define ASM_RET	"ret; int3\n\t"
#else /* !CONFIG_SLS */
#define ASM_RET	"ret\n\t"
#endif /* CONFIG_RETHUNK */

#endif /* __ASSEMBLY__ */

#endif /* _ASM_X86_LINKAGE_H */

